//
//  AASDirectorySubscription.h
//
//  Created by andreas on Fri Nov 16 2001.
//  Copyright (c) 2001-2009 Andreas Schwarz @ immaterial.
//
//  OS X 10.5 or later required.
//
//  Most interesting in this header file is probably the subscriber notification protocol at the bottom.  Don't miss it.
//

#import <Foundation/Foundation.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/event.h>
#include <sys/time.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <fts.h>







// THIS PROTOCOL IS IMPORTANT

/* Subscribers must implement this method.  It is called whenever the directory changes. It gives an array of dictionaries containing information about the changed files.  The keys to these dictionaries are below. If the AASDirectorySubscription is set to load asynchronously, this is called periodically while the files load, giving AASFileAppeared events with the newly loaded files.  To check if the AASDirectorySubscription is currently loading, use the isLoading method.  After all loading has finished, the AASDirectorySubscription will check if any changes happened during loading, and if so will call this with the appropriate changes, or will call it with an empty changes array (so that if the subscriber needs to check isLoading each time the method is called to see when the loading finishes, it will have a chance to get NO from isLoading at an appropriate time). */

@protocol AASDirectorySubscriptionProtocol <NSObject>
- (void)directorySubscription:(AASDirectorySubscription *)subscription foundChanges:(NSArray *)changes ;
@end






////////////// Dictionary key strings ////////////////

/* Key for the file event string
	The file event constants (directly below) identify what sort of file event occurred for the file.  The file event key will return an NSNumber object with one of these values. */
extern NSString *AASFileEventKey;
    typedef enum {
    	kAASFileMovedEvent			= 1,	// The file moved out of the directory (may also be renamed).
		
    	kAASFileRenamedEvent		= 2,	// The file was renamed but remains in the directory.
		
    	kAASFileRemovedEvent		= 4,	// The file was deleted or is otherwise untrackable. The file should return NO for -isValid.

    	kAASFileUpdatedEvent		= 8,	// File data or metadata updated, but no moves or renames.
		
    	kAASFileAppearedEvent		= 16,	// This new file appeared in the parent directory.
		
    	kAASFileMovedInParentEvent	= 32	// The file "moved" (its path changed) but it is still within its parent directory. This
    } AASFileEvents;						// happens when the parent or a grandparent directory moved. When you get this notification
											// it is possible the file has also been renamed.
											
    
/* Key for the file object for this event
    The file object is of whatever class was specified when the AASDirectorySubscription was created (which would be the class that you set for AASFileManager if you are using it). */
extern NSString *AASFileKey;





@class AASFile;
@class AASFileLoadOperation;

@interface AASDirectorySubscription : NSObject {
    NSMapTable *subscribers;
	CFURLRef lastKnownURL;
	AASFile *directoryFile;
    NSMapTable *directoryInfo;
    NSMapTable *directoryInfoDirectoriesOnly;
	
	AASFileLoadOperation *currentLoadOperation;
	NSLock *asyncLoadingLock;
	CFRunLoopTimerRef asyncLoadingCheckTimer;
    NSMutableArray *asyncLoadingQueue;
    BOOL stopLoading;

    Class fileClass;
    
    BOOL asynchronous;
    BOOL checkDirContents;
    
    BOOL shouldCheck;
}





/* Inits the AASDirectorySubscription for a particular subscriber and a particular directory.  This is meant to be done by an AASFileManager instance, but it can be used independently if you want to for some reason.  When asked to check the directory, it checks the kernel queue to see if anything has changed since the last check, and if so, sends the subscriber(s) an array with information on what files changed.  This tracks the directory's immediate contents only, it doesn't recursively check subdirectories. Returns nil if init fails for any reason.  If asynchronous is YES, the contents of the directory are loaded and the file objects created in a separate thread, and the contents are checked for changes in a separate thread; the initial return from methods like -files may be empty arrays or only some of the actual directory contents. Subscribers will periodically recieve AASFileAppeared notifications from the main thread to update while asynchronous loading is occurring. Subscribers can check the -isLoading method during notifications if desired; a final notification will be sent when loading has completed, so checking -isLoading during the notifications will have an opportunity to return NO. */
- (id)initForSubscriber:(id)sub withFile:(AASFile *)fObj fileClass:(Class)fClass asynchronous:(BOOL)async;

/* Calls initForSubscriber:withFile:fileClass: with the default AASFile class. */
- (id)initForSubscriber:(id)sub withFile:(AASFile *)fObj asynchronous:(BOOL)async;

/* Calls initForSubscriber:withFile:fileClass: with the default AASFile class and NO for asynchronous. */
- (id)initForSubscriber:(id)sub withFile:(AASFile *)fObj;





/* Returns the number of subscribers the reciever has.  The reciever should only be released if subscribers == 0 (This is typically handled by the AASFileManager instance). */
- (int)numberOfSubscribers;

/* Adds a subscriber.  The subscriber is not retained, so that the subscriber may unsubscribe itself in its dealloc method.  For obvious reasons, it is important the subscriber make sure it is unsubscribed before or during its deallocation. Do not subscribe to the same directory multiple times with the same subscriber. */
- (void)subscribe:(id <AASDirectorySubscriptionProtocol>)subscriber;

/* Removes a subscriber (if it exists), returns the number of subscribers left. ALWAYS unsubscribe before releasing the subscriber object (you can unsubscribe in the subscriber's dealloc method). Only removes the subscriber once, so don't subscribe multiple times to one AASDirectorySubscription with the same subscriber. */
- (int)unsubscribe:(id <AASDirectorySubscriptionProtocol>)subscriber;






/* Returns the path of the directory whose contents this AASDirectorySubscription is watching. */
- (NSString *)path;

/* Returns the url of the directory this AASDirectorySubscription is watching. */
- (NSURL *)url;



/* Tells whether the reciever is still valid or not (it isn't valid anymore if the directory it is checking was deleted). */
- (BOOL)isValid;




/* Tells whether or not the AASDirectorySubscription is currently loading files from disk (only happens if asynchronous was set to YES). */
- (BOOL)isLoading;

/* If the AASDirectorySubscription is set to load asynchronously and hasn't begun or hasn't completed loading the directory's contents, this stops the asynchronous process and finishes the loading immediately. Everyone's in a hurry these days. */
- (void)loadImmediately;






/* Checks if there have been any changes to the directory or the files in it.  If the directory itself is deleted or otherwise lost, the subscription becomes invalid and checking no longer occurs; the subscriber should watch for a deletion of the directory it subscribed to and unsubscribe itself if that happens.  Validity can also be checked through the -isValid method.  If a directory is moved or renamed, the AASFileMovedEvent/AASFileRenamedEvent will be reported for the directory, AND for all files that remain within it.  The checkDirectory method has been split into two parts so that the AASFileManager can make sure all directories are checked (and all file objects that need updating are updated) before any of the subscribers recieve notifications.  This method however just causes a check and notifications to be sent immediately. */
- (void)checkDirectory;

/* Checks the directory (as explained above), making the array pointer point to an array of changes if there are any changes, or nil otherwise.  If non-nil, the array given must be released afterwards (it is not autoreleased).  Typically used in conjunction with the notifySubscribers: method. */
- (void)checkDirectory:(NSArray **)array;


/* Notifies its subscribers about the specified changes. It is safe for a subscriber to unsubscribe itself (and/or cause other subscribers to unsubscribe themselves) during this method call. */
- (void)notifySubscribers:(NSArray *)changes;





/* Returns an array of file objects for all the visible files in the directory. */
- (NSArray *)files;

/* Returns the file object for the directory whose contents this AASDirectorySubscription is watching. */
- (id)directoryFile;


@end






