//
//  AASFileManagerTracking.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/3/08.
//  Copyright (c) 2001-2009 Andreas Schwarz @ immaterial.
//
//  OS X 10.5 or later required.

#import <Cocoa/Cocoa.h>
#import "AASFileManager.h"

@protocol AASDirectorySubscriptionProtocol;

/* This AASFileManager category contains methods for tracking changes to files on disk. This is one of the most useful functions of the AASFilesystemTools framework. Tracking and updates is accomplished using kernel queues rather than constantly scanning the filesystem directly. */


@interface AASFileManager (FileTracking)


/* The following set: method must be called before any subscriber asks to track a directory. If you do this correctly, the methods will return YES. If you attempt to set them afterward, they will return NO. */

/* Sets whether or not to read files from the filesystem in separate threads. Subscribers will be notified of changes on the main thread (see the AASDirectorySubscription header for more). Default is NO. */
- (BOOL)setAsynchronous:(BOOL)async;
/* Returns whether or not the AASFileManager instance is set to load asynchronously or not. */
- (BOOL)asynchronous;




/* The following set: methods may be called at any point. */

/* Changes the time interval (measured in seconds) at which the kernel event queue is checked for changes.  Do not ever set it to "0" - if you want to stop tracking, use the setPaused: method.  For example, you might set the time interval to 4 seconds while your app is in the foreground, 6.0 while it is in the background but unhidden, and pause it with setPaused: when it is hidden.  This can be accomplished by watching for NSApplication's NSApplicationDidHideNotification, NSApplicationDidResignActiveNotification, and other related notifications (usually in your app's delegate). Default is 3.0. */
- (void)setPollingInterval:(NSTimeInterval)interval;
/* Returns the time interval at which the file system is currently being checked (though checking still might be paused via setPaused:). The default time interval is 3.0 seconds. */
- (float)pollingInterval;

/* Pauses or unpauses filesystem tracking by the AASFileManager.  While it is paused, no tracking goes on.  See setPollingInterval: for further comments on when this would be useful. Default is NO (but no tracking occurs until the AASFileManager has at least one subscriber). */
- (void)setPaused:(BOOL)isPaused;
/* Returns whether or not filesystem tracking is currently paused. */
- (BOOL)isPaused;
/* Convenience methods - they merely call the setPaused: method with a YES or NO. */
- (void)pause;
- (void)unPause;




/* Subscribing and unsubscribing to file tracking services. */


/* Subscribes a particular object, adding the directory represented by "file" to the list of directories to check.  Returns the AASDirectorySubscription object that the subscriber subscribed to, or nil if it doesn't succeed.  The subscriber doesn't need to keep a reference to the AASDirectorySubscription object, though it might be useful for various reasons.  The returned object should never be released by the subscriber!  Unsubscribing will release it if necessary (there may be other subscribers to that AASDirectorySubscription instance). Do not unsubscribe through the returned object if you have subscribed through the AASFileManager, rather unsubscribe through the AASFileManager so it can properly remove the subscriber from its own lists. */
- (AASDirectorySubscription *)subscribe:(id <AASDirectorySubscriptionProtocol>)subscriber forFile:(AASFile *)file;
/* Same as subscribe: forFile: but can be passed a path instead of a file object. */
- (AASDirectorySubscription *)subscribe:(id <AASDirectorySubscriptionProtocol>)subscriber forPath:(NSString *)path;
/* Same as subscribe: forFile: but can be passed an NSURL instead of a file object. */
- (AASDirectorySubscription *)subscribe:(id <AASDirectorySubscriptionProtocol>)subscriber forURL:(NSURL *)url;

/* Unsubscribes a subscriber from all subscriptions it may have.  If you want to unsubscribe given a certain subscription or path, see the next few methods.  This is much slower than the unsubscribe:forSubscription: and unsubscribe:forPath: methods so avoid them if possible. */
- (void)unsubscribe:(id <AASDirectorySubscriptionProtocol>)unsubscriber;
/* Unsubscribes a subscriber from the subscription given.  This is the most efficient way to unsubscribe, so use it when possible.  If you want to unsubscribe from all subscriptions, you can either use the unsubscribe: method or call this method (or the unsubscribe:forPath: method) for each subscription (which is more efficient, if you can do it). */
- (void)unsubscribe:(id <AASDirectorySubscriptionProtocol>)unsubscriber forSubscription:(AASDirectorySubscription *)sub;
/* Unsubscribes a subscriber from the subscription for the file object given.  If you want to unsubscribe from all subscriptions, you can either use the unsubscribe: method or call this method (or the unsubscribe:forSubscription: method) for each directory. */
- (void)unsubscribe:(id <AASDirectorySubscriptionProtocol>)unsubscriber forFile:(AASFile *)file;
/* Same as unsubscribe:forFile: but takes a path string. */
- (void)unsubscribe:(id <AASDirectorySubscriptionProtocol>)unsubscriber forPath:(NSString *)path;
/* Same as unsubscribe:forFile: but takes an NSURL. */
- (void)unsubscribe:(id <AASDirectorySubscriptionProtocol>)unsubscriber forURL:(NSURL *)url;

/* Checks all the AASDirectorySubscription instances to see if there's already an instance for this file object.  Returns it if it exists, returns nil otherwise.  This is mainly for internal use. DO NOT rely on the subscription to keep itself updated and DO NOT subscribe to the subscription directly through this returned instance. If you want to keep updated on a certain directory, subscribe through one of the above subscribe: methods. */
- (AASDirectorySubscription *)subscriptionForFile:(AASFile *)file;


/* Checking the filesystem */

/* This gets called periodically by a timer to tell all the AASDirectorySubscription instances to check the kernel queue for updates.  Additionally, you may call it yourself (pass it nil or whatever you like as sender) if you want to.  For example, call this after your program performs any kind of filesystem change (copying/moving/renaming/creating/whatever a file) in order to update itself immediately. File operations perfomed by the AASFileManager (see the AASFileManagerOperations category) automatically call this method. */
- (void)checkFilesystem:(id)sender;



@end




